# Copyright 2018-present Lenovo
# Confidential and Proprietary


"""
Example:
python datasource_process.py
    --location='/home/hpcadmin/ai_stdio/datasource/test'
    --label='dog' --image_path='/home/hpcadmin/test.zip'
"""

import argparse
import os
import shutil
import tarfile
import uuid
import zipfile

from PIL import Image
from six import print_


def convert_and_generate_thumbnail(
        image_path, origin_dir, thumb_dir):
    success_num = 0
    failed_num = 0
    if not os.path.isdir(image_path):
        print_('[UPLOAD_ERROR]: The image path is incorrect')
        return success_num, failed_num
    try:
        for root, dirs, files in os.walk(image_path, topdown=False):
            failed_num += len(files)
            for file_name in files:
                srcfile = os.path.join(root, file_name)
                ext = os.path.splitext(srcfile)[-1]
                if ext.lower() in accepteble_file_pattens:
                    file_id = str(uuid.uuid1())
                    dstfile = os.path.join(origin_dir, file_id + '.jpg')
                    flag1 = copy_or_convert_image(
                        srcfile, ext.lower(), dstfile, image_path)
                    if flag1:
                        flag2 = generate_thumbnail(
                            thumb_dir, dstfile, srcfile, image_path)
                        if flag2:
                            success_num += 1
                            failed_num -= 1
    except Exception as e:
        print_("[UPLOAD_ERROR]: Convert picture error, reason: %s" % str(e))
    return success_num, failed_num


def copy_or_convert_image(srcfile, ext, dstfile, image_path):
    pic_path = srcfile.replace(image_path + '/', '')
    try:
        if ext == '.jpg':
            shutil.copyfile(srcfile, dstfile)
        else:
            with Image.open(srcfile) as image:
                if image.mode in ['P', 'RGBA']:
                    image.convert('RGB').save(dstfile)
                else:
                    image.save(dstfile)
        print_('[CONVERT]: Convert %s successfully.' % pic_path)
        return True
    except Exception:
        if os.path.exists(dstfile):
            os.remove(dstfile)
        print_('[UPLOAD_ERROR]: The image format '
               'cannot be converted .JPG : %s.' % pic_path)
    return False


def generate_thumbnail(thumb_dir, dstfile, srcfile, image_path):
    """Generate thumbnails"""
    pic_path = srcfile.replace(image_path + '/', '')
    thumbnail_size = [104, 74]
    filepath, tempfilename = os.path.split(dstfile)
    picturename, ex = os.path.splitext(tempfilename)
    save_path = os.path.join(thumb_dir, picturename + ".gif")
    try:
        with Image.open(dstfile) as image:
            # Generate thumbnail
            image.thumbnail(thumbnail_size, Image.ANTIALIAS)
            # Add black background
            b = Image.new(mode="RGBA", size=thumbnail_size)
            b.paste(image, ((b.size[0] - image.size[0]) / 2,
                            (b.size[1] - image.size[1]) / 2))
        b.save(save_path, "GIF")
        print_('[THUMBNAIL]:  %s thumbnail created successfully!' % pic_path)
        with open(record_file_path, "a+") as f:
            f.write(dstfile + '\n')
            f.flush()
        return True
    except Exception:
        if os.path.exists(dstfile):
            os.remove(dstfile)
        if os.path.exists(save_path):
            os.remove(save_path)
        print_('[UPLOAD_ERROR]: Thumbnail creation failed : %s ' % pic_path)
    return False


def parse_args():
    """Parse input arguments."""
    parser = argparse.ArgumentParser(description='Datasource process script')
    parser.add_argument('--location', dest='location',
                        help='Datasource project path',
                        required=True, type=str)
    parser.add_argument('--image_path', dest='image_path',
                        help='Upload pictures path',
                        required=True, type=str)
    parser.add_argument('--label', dest='label',
                        help='Image classification label',
                        default='')

    return parser.parse_args()


def get_temp_dir(location, temp_name):
    dest_dir = os.path.join(location, 'temp', temp_name)
    temp_dir = os.path.join(location, 'temp')
    if not os.path.exists(temp_dir):
        os.mkdir(temp_dir)
    if not os.path.exists(dest_dir):
        os.mkdir(dest_dir)
    return dest_dir


def upload_pics(image_path, temp_name):
    try:
        dest_dir = get_temp_dir(location, temp_name)
        if os.path.isdir(image_path):
            shutil.rmtree(dest_dir)
            shutil.copytree(image_path, dest_dir)
        elif zipfile.is_zipfile(image_path):
            print_("Decompress zip file...")
            with zipfile.ZipFile(image_path) as zip:
                for name in zip.namelist():
                    zip.extract(name, path=dest_dir)
        elif tarfile.is_tarfile(image_path):
            print_("Decompress tar file...")
            with tarfile.open(image_path) as tar:
                tar.extractall(path=dest_dir)
        elif os.path.isfile(image_path):
            shutil.copy(image_path, dest_dir)
        else:
            print_("[UPLOAD_ERROR]: Unsupported image"
                   " path %s, please check!" % (image_path))
            exit(1)
    except Exception as e:
        print_("[UPLOAD_ERROR]: Upload picture error" + str(e))
    return dest_dir


if __name__ == '__main__':
    args = parse_args()
    location = args.location
    image_path = args.image_path
    label = args.label
    temp_name = str(uuid.uuid1())

    accepteble_file_pattens = {'.jpg', '.png', '.jpeg', '.bmp', '.gif'}

    print_("Run datasource process script start...")
    if not all([location, image_path, temp_name]):
        print_("[UPLOAD_ERROR]: Args error, please check!")
        exit(1)

    if label:
        origin_dir = os.path.join(location, "origin", label)
        thumbnail_dir = os.path.join(location, "thumbnail", label)
        record_file_path = os.path.join(
            location, "thumbnail", label + "_record_file.txt")
    else:
        origin_dir = os.path.join(location, "unlabel")
        thumbnail_dir = os.path.join(location, "thumbnail")
        record_file_path = os.path.join(location, "record_file.txt")
    try:
        dest_dir = upload_pics(image_path, temp_name)

        print_("Begin convert picture and generate thumbnail...")
        success_num, failed_num = \
            convert_and_generate_thumbnail(dest_dir, origin_dir, thumbnail_dir)
        print_("[UPLOAD_FINISH]: The successful upload statistic is %d."
               "The failed upload statistic is %d." % (success_num, failed_num))
    except Exception:
        print_("[UPLOAD_ERROR]: Upload picture error")
    finally:
        dest_dir = os.path.join(location, 'temp', temp_name)
        if os.path.exists(dest_dir):
            shutil.rmtree(dest_dir)
